@@ -1,6 +1,6 @@
 /*
     QMPlay2 is a video and audio player.
-    Copyright (C) 2010-2024  Błażej Szczygieł
+    Copyright (C) 2010-2025  Błażej Szczygieł
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -93,7 +93,10 @@ void DemuxerThr::loadImage()
     if (isDemuxerReady())
     {
         const QByteArray demuxerImage = demuxer->image();
-        QImage img = QImage::fromData(demuxerImage);
+        QImage img = demuxerImage.isEmpty()
+            ? QImage()
+            : QImage::fromData(demuxerImage)
+        ;
         if (!img.isNull())
             emit QMPlay2Core.coverDataFromMediaFile(demuxerImage);
         else
@@ -465,7 +468,7 @@ void DemuxerThr::run()
     const int minBuffered = demuxer->dontUseBuffer()
         ? 1
         : localStream
-          ? qMax(1, minBuffSizeLocal)
+          ? qMax(10, minBuffSizeLocal)
           : 0 // Use "forwardTime"
     ;
     const double forwardTime = (minBuffered == 0)
@@ -531,8 +534,7 @@ void DemuxerThr::run()
 
     QHash<int, int> recStreamsMap;
 
-    if (unknownLength)
-        emit allowRecording(true);
+    emit allowRecording(true);
 
     auto handleRecording = [&](bool startAndStop) {
         if (startAndStop && m_recording && !m_recMuxer)
@@ -787,17 +789,20 @@ void DemuxerThr::startRecordingInternal(QHash<int, int> &recStreamsMap)
     if (m_recMuxer->isOk())
     {
         emit recording(true, false, fileName);
-        auto writePackets = [&](PacketBuffer &packets, int packetsStreamIdx) {
-            if (int recStreamIdx = recStreamsMap.value(packetsStreamIdx, -1); recStreamIdx > -1)
-            {
-                packets.iterate([&](const Packet &packet) {
-                    m_recMuxer->write(packet, recStreamIdx);
-                });
-            }
-        };
-        writePackets(playC.vPackets, playC.videoStream);
-        writePackets(playC.aPackets, playC.audioStream);
-        writePackets(playC.sPackets, playC.subtitlesStream);
+        if (unknownLength)
+        {
+            auto writePackets = [&](PacketBuffer &packets, int packetsStreamIdx) {
+                if (int recStreamIdx = recStreamsMap.value(packetsStreamIdx, -1); recStreamIdx > -1)
+                {
+                    packets.iterate([&](const Packet &packet) {
+                        m_recMuxer->write(packet, recStreamIdx);
+                    });
+                }
+            };
+            writePackets(playC.vPackets, playC.videoStream);
+            writePackets(playC.aPackets, playC.audioStream);
+            writePackets(playC.sPackets, playC.subtitlesStream);
+        }
     }
     else
     {
