@@ -1,6 +1,6 @@
 /*
     QMPlay2 is a video and audio player.
-    Copyright (C) 2010-2024  Błażej Szczygieł
+    Copyright (C) 2010-2025  Błażej Szczygieł
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -25,6 +25,10 @@
 #include <Settings.hpp>
 #include <Packet.hpp>
 
+#ifdef Q_OS_ANDROID
+#   include <QFile>
+#endif
+
 #include <limits>
 
 extern "C"
@@ -36,6 +40,42 @@ extern "C"
     #include <libavutil/pixdesc.h>
 }
 
+#ifdef Q_OS_ANDROID
+static int readPacketQFile(void *opaque, uint8_t *buf, int bufSize)
+{
+    auto &f = *static_cast<QFile *>(opaque);
+    const int64_t bread =f.read(reinterpret_cast<char *>(buf), bufSize);
+    if (bread == 0)
+        return AVERROR_EOF;
+    return bread;
+}
+static int64_t seekQFile(void *opaque, int64_t offset, int whence)
+{
+    auto &f = *static_cast<QFile *>(opaque);
+    switch (whence)
+    {
+        case SEEK_SET:
+            if (!f.seek(offset))
+                return -1;
+            break;
+        case SEEK_CUR:
+            if (offset != 0)
+            {
+                if (!f.seek(f.pos() + offset))
+                    return -1;
+            }
+            break;
+        case SEEK_END:
+            if (!f.seek(f.size() - offset))
+                return -1;
+            break;
+        case AVSEEK_SIZE:
+            return f.size();
+    }
+    return f.pos();
+}
+#endif
+
 static void matroska_fix_ass_packet(AVRational stream_timebase, AVPacket *pkt)
 {
     AVBufferRef *line;
@@ -196,14 +236,15 @@ private:
 
 /**/
 
-FormatContext::FormatContext(bool reconnectStreamed) :
+FormatContext::FormatContext(bool reconnectNetwork, bool allowExperimental) :
     isError(false),
     currPos(0.0),
     abortCtx(new AbortContext),
     formatCtx(nullptr),
     packet(nullptr),
     oggHelper(nullptr),
-    reconnectStreamed(reconnectStreamed),
+    m_reconnectNetwork(reconnectNetwork),
+    m_allowExperimental(allowExperimental),
     isPaused(false), fixMkvAss(false),
     isMetadataChanged(false),
     lastTime(0.0),
@@ -221,7 +262,7 @@ FormatContext::~FormatContext()
         av_packet_free(&packet);
     }
     delete oggHelper;
-    for (StreamInfo *streamInfo : qAsConst(streamsInfo))
+    for (StreamInfo *streamInfo : std::as_const(streamsInfo))
         delete streamInfo;
 }
 
@@ -328,14 +369,7 @@ QList<QMPlay2Tag> FormatContext::tags() const
     }
     if (isStreamed && !(value = getTag(formatCtx->metadata, "StreamTitle", false)).isEmpty())
     {
-        int idx = value.indexOf(" - ");
-        if (idx < 0)
-            tagList += {QString::number(QMPLAY2_TAG_TITLE), value};
-        else
-        {
-            tagList += {QString::number(QMPLAY2_TAG_TITLE), value.mid(idx + 3)};
-            tagList += {QString::number(QMPLAY2_TAG_ARTIST), value.mid(0, idx)};
-        }
+        tagList += {QString::number(QMPLAY2_TAG_TITLE), value};
     }
     else if (AVDictionary *dict = getMetadata())
     {
@@ -696,7 +730,15 @@ bool FormatContext::open(const QString &_url, const QString &param)
 
     AVInputFormat *inputFmt = nullptr;
     if (scheme == "file")
+    {
         isLocal = true;
+    }
+#ifdef Q_OS_ANDROID
+    else if (scheme == "content")
+    {
+        isLocal = true;
+    }
+#endif
     else
     {
         if (scheme != "rtsp")
@@ -713,14 +755,32 @@ bool FormatContext::open(const QString &_url, const QString &param)
     if (!inputFmt)
     {
         url = Functions::prepareFFmpegUrl(_url, options, false);
-        if (!isLocal && reconnectStreamed)
+        if (!isLocal && m_reconnectNetwork)
+        {
+            av_dict_set(&options, "reconnect", "1", 0);
             av_dict_set(&options, "reconnect_streamed", "1", 0);
+            av_dict_set(&options, "reconnect_on_network_error", "1", 0);
+            av_dict_set(&options, "reconnect_delay_max", "7", 0);
+        }
     }
 
     formatCtx = avformat_alloc_context();
+    if (m_allowExperimental)
+    {
+        formatCtx->strict_std_compliance = FF_COMPLIANCE_EXPERIMENTAL;
+    }
     formatCtx->interrupt_callback.callback = (int(*)(void *))interruptCB;
     formatCtx->interrupt_callback.opaque = &abortCtx->isAborted;
 
+#ifdef Q_OS_ANDROID
+    if (isLocal && oggOffset < 0)
+    {
+        m_file = std::make_unique<QFile>(url);
+        m_file->open(QFile::ReadOnly);
+        formatCtx->pb = avio_alloc_context(nullptr, 0, false, m_file.get(), readPacketQFile, nullptr, seekQFile);
+    }
+#endif
+
     if (oggOffset >= 0)
     {
         oggHelper = new OggHelper(url, oggTrack, oggSize, formatCtx->interrupt_callback);
@@ -742,12 +802,27 @@ bool FormatContext::open(const QString &_url, const QString &param)
     if (!formatCtx || disabledDemuxers.contains(name()))
         return false;
 
+    auto isAvif = [&] {
+        if (formatCtx->nb_streams != 1 || !name().contains("mp4"))
+            return false;
+
+        AVStream *stream = formatCtx->streams[0];
+        if (stream->codecpar->codec_id != AV_CODEC_ID_AV1 || stream->nb_frames != 1)
+            return false;
+        if (stream->time_base.num != 1 || stream->time_base.den != 1)
+            return false;
+        if (stream->avg_frame_rate.num != 1 || stream->avg_frame_rate.den != 1)
+            return false;
+
+        return true;
+    };
+
     if (name() == "sdp")
     {
         isLocal = false;
         formatCtx->flags |= AVFMT_FLAG_NOBUFFER;
     }
-    else if (name().startsWith("image2") || name().endsWith("_pipe"))
+    else if (name().startsWith("image2") || name().endsWith("_pipe") || isAvif())
     {
         if (!settings.getBool("StillImages"))
             return false;
@@ -810,6 +885,8 @@ bool FormatContext::open(const QString &_url, const QString &param)
     streamsOffset.resize(formatCtx->nb_streams);
     nextDts.resize(formatCtx->nb_streams);
 
+    const bool hasTeletextCodec = avcodec_find_decoder(AV_CODEC_ID_DVB_TELETEXT);
+    std::vector<StreamInfo *> subtitlesStreams;
     int nVideoStreams = 0;
     int nDefaultVideoStreams = 0;
     for (unsigned i = 0; i < formatCtx->nb_streams; ++i)
@@ -831,6 +908,12 @@ bool FormatContext::open(const QString &_url, const QString &param)
                     ++nDefaultVideoStreams;
                 ++nVideoStreams;
             }
+
+            else if (streamInfo->params->codec_type == AVMEDIA_TYPE_SUBTITLE)
+            {
+                if (hasTeletextCodec)
+                    subtitlesStreams.push_back(streamInfo);
+            }
         }
         if (!fixMkvAss && formatCtx->streams[i]->codecpar->codec_id == AV_CODEC_ID_ASS && !strncasecmp(formatCtx->iformat->name, "matroska", 8))
             fixMkvAss = true;
@@ -841,6 +924,12 @@ bool FormatContext::open(const QString &_url, const QString &param)
     }
     if (streamsInfo.isEmpty())
         return false;
+    if (subtitlesStreams.size() == 1 && subtitlesStreams[0]->params->codec_id == AV_CODEC_ID_DVB_TELETEXT)
+    {
+        // Don't skip teletext if we have decoder and it's the only stream
+        Q_ASSERT(hasTeletextCodec);
+        subtitlesStreams[0]->skip_auto_select = false;
+    }
 
     isOneStreamOgg = (name() == "ogg" && streamsInfo.count() == 1); //Workaround for OGG network streams
 
@@ -897,7 +986,7 @@ bool FormatContext::open(const QString &_url, const QString &param)
                 {
                     if (firstAudioIdx > -1)
                     {
-                        for (auto &&streamInfo : qAsConst(streamsInfo))
+                        for (auto &&streamInfo : std::as_const(streamsInfo))
                         {
                             if (streamInfo->params->codec_type == AVMEDIA_TYPE_AUDIO)
                                 streamInfo->is_default = false;
@@ -930,7 +1019,7 @@ void FormatContext::setStreamOffset(double offset)
 void FormatContext::selectStreams(const QSet<int> &selectedStreams)
 {
     m_allDiscarded = true;
-    for (AVStream *stream : qAsConst(streams))
+    for (AVStream *stream : std::as_const(streams))
     {
         if (stream->codecpar->codec_type == AVMEDIA_TYPE_DATA || stream->codecpar->codec_type == AVMEDIA_TYPE_ATTACHMENT)
         {
@@ -981,8 +1070,14 @@ StreamInfo *FormatContext::getStreamInfo(AVStream *stream) const
         streamInfo->must_decode = false;
     }
 
+    if (streamInfo->params->codec_id == AV_CODEC_ID_DVB_TELETEXT)
+    {
+        streamInfo->skip_auto_select = true;
+    }
+
     streamInfo->is_default = stream->disposition & AV_DISPOSITION_DEFAULT;
     streamInfo->time_base = stream->time_base;
+    streamInfo->ts_discont_possible = (formatCtx->iformat->flags & AVFMT_TS_DISCONT);
 
     if (streamInfo->params->codec_type != AVMEDIA_TYPE_ATTACHMENT)
     {
