@@ -1,6 +1,6 @@
 /*
     QMPlay2 is a video and audio player.
-    Copyright (C) 2010-2024  Błażej Szczygieł
+    Copyright (C) 2010-2025  Błażej Szczygieł
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -124,7 +124,7 @@ template<typename T>
 void Module::setInstance()
 {
     QMutexLocker locker(&mutex);
-    for (ModuleCommon *mc : qAsConst(instances))
+    for (ModuleCommon *mc : std::as_const(instances))
     {
         T *t = dynamic_cast<T *>(mc);
         if (t)
@@ -134,7 +134,7 @@ void Module::setInstance()
 
 /**/
 
-#define QMPLAY2_MODULES_API_VERSION 27
+#define QMPLAY2_MODULES_API_VERSION 29
 
 #define QMPLAY2_EXPORT_MODULE(ModuleClass) \
     extern "C" Q_DECL_EXPORT quint32 getQMPlay2ModuleAPIVersion() \
