@@ -1,6 +1,6 @@
 /*
     QMPlay2 is a video and audio player.
-    Copyright (C) 2010-2024  Błażej Szczygieł
+    Copyright (C) 2010-2025  Błażej Szczygieł
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -121,9 +121,27 @@ void YouTubeDL::addr(const QString &url, const QString &param, QString *streamUr
     if (!streamUrl && !name)
         return;
 
+    QByteArray refererHeader;
+
     QStringList paramList {"-e"};
     if (!param.isEmpty())
-        paramList << "-f" << param;
+    {
+        if (!QUrl(param).host().isEmpty())
+        {
+            refererHeader = "Referer: " + param.toUtf8() + "\r\n";
+        }
+        paramList << (refererHeader.isEmpty() ? "-f" : "--referer") << param;
+    }
+    else
+    {
+        auto &QMPSettings = QMPlay2Core.getSettings();
+        if (QMPSettings.getBool("YtDl/DefaultQualityEnabled"))
+        {
+            auto dq = QMPSettings.getString("YtDl/DefaultQuality").simplified();
+            if (!dq.isEmpty())
+                paramList << "-f" << std::move(dq);
+        }
+    }
     QStringList ytdlStdout = exec(url, paramList, err);
     if (ytdlStdout.isEmpty())
         return;
@@ -142,6 +160,11 @@ void YouTubeDL::addr(const QString &url, const QString &param, QString *streamUr
                 *streamUrl += "[" + tmpUrl + "]";
             *streamUrl += "}";
         }
+        if (!refererHeader.isEmpty())
+        {
+            for (const QString &url : std::as_const(ytdlStdout))
+                QMPlay2Core.addRawHeaders(url, refererHeader);
+        }
     }
     if (name && !title.isEmpty())
         *name = title;
