@@ -1,6 +1,6 @@
 /*
     QMPlay2 is a video and audio player.
-    Copyright (C) 2010-2024  Błażej Szczygieł
+    Copyright (C) 2010-2025  Błażej Szczygieł
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published
@@ -1024,7 +1024,7 @@ void Window::ensureVideoPipeline()
     const int vkImgH = vkImage->size().height;
     const int imgH = m_imgSize.height();
 
-    const float subtract = m_frame.hasBorders()
+    const float subtract = (m_nearestScaling || m_frame.hasBorders())
         ? 0.0f
         : 1.0f
     ;
